package main

import (
  "fmt"
  "time"

  "k8s.io/api/apps/v1"
  "k8s.io/apimachinery/pkg/labels"
  "k8s.io/client-go/informers"
  "k8s.io/client-go/kubernetes"
  "k8s.io/client-go/tools/cache"
  "k8s.io/klog/v2"
  "sigs.k8s.io/controller-runtime/pkg/client/config"
)

func AddFunc(obj interface{}) {
  deployment := obj.(*v1.Deployment)
  klog.Infof("add a deployment: %#v", deployment)
}

func UpdateFunc(oldObj, newObj interface{}) {
  oldDeployment := oldObj.(*v1.Deployment)
  newDeployment := newObj.(*v1.Deployment)

  klog.Infof("update a oldDeployment: %#v, newDeployment: %#v", oldDeployment, newDeployment)
}

func DeleteFunc(obj interface{}) {
  deployment := obj.(*v1.Deployment)
  klog.Infof("delete a deployment: %#v", deployment.Name)
}

func main() {
  conf, err := config.GetConfig()
  if err != nil {
    fmt.Printf("`~/.kube/config` not found, exit 1")
    return
  }

  clientSet, err := kubernetes.NewForConfig(conf)
  if err != nil {
    fmt.Printf("get clientSet err %#v, skip", err)
    return
  }

  factory := informers.NewSharedInformerFactory(clientSet, 30*time.Second)
  deploymentInformer := factory.Apps().V1().Deployments().Informer()
  deploymentInformer.AddEventHandler(cache.ResourceEventHandlerFuncs{
    AddFunc:    AddFunc,
    UpdateFunc: UpdateFunc,
    DeleteFunc: DeleteFunc,
  })

  deploymentLister := factory.Apps().V1().Deployments().Lister()

  stopCh := make(chan struct{})
  defer close(stopCh)
  // start Informer List and Watch
  factory.Start(stopCh)

  // wait for all cache sync
  factory.WaitForCacheSync(stopCh)

  deployments, err := deploymentLister.Deployments("default").List(labels.Everything())

  // print Deployment List
  for index, dp := range deployments {
    fmt.Printf("%d: %#v\n", index, dp)
  }

  <-stopCh
}

// test case:
// kubectl create deployment nginx --image=nginx
